/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ------------------------------------------------------------------------- */

#include "..\wrapper\wrapper.h"

/* ----------------------------------------------------------------------------
 * Function     : configure
 * ----------------------------------------------------------------------------
 * Description  : Implements the configure method defined in "wrapper.h" 
 * Inputs       : refer "wrapper.h" 
 * Outputs      : refer "wrapper.h" 
 * Assumptions  : refer "wrapper.h" 
 * ------------------------------------------------------------------------- */
void configure(pBaseDSPCodec /* unused */)
{
    /* Nothing required here, not used in ECHO codec */
}

/* ----------------------------------------------------------------------------
 * Function     : encodeReset
 * ----------------------------------------------------------------------------
 * Description  : Implements the encodeReset method defined in "wrapper.h" 
 * Inputs       : refer "wrapper.h" 
 * Outputs      : refer "wrapper.h" 
 * Assumptions  : refer "wrapper.h" 
 * ------------------------------------------------------------------------- */
void encodeReset(pBaseDSPCodec /* unused */)
{
    /* Nothing required here, not used in ECHO codec */
}

/* ----------------------------------------------------------------------------
 * Function     : decodeReset
 * ----------------------------------------------------------------------------
 * Description  : Implements the decodeReset method defined in "wrapper.h" 
 * Inputs       : refer "wrapper.h" 
 * Outputs      : refer "wrapper.h" 
 * Assumptions  : refer "wrapper.h" 
 * ------------------------------------------------------------------------- */
void decodeReset(pBaseDSPCodec /* unused */)
{
    /* Nothing required here, not used in ECHO codec */
}

/* ----------------------------------------------------------------------------
 * Function     : encode
 * ----------------------------------------------------------------------------
 * Description  : Implements the encode method defined in "wrapper.h" 
 *                Specifically, this just copies the input to the output 
 * Inputs       : refer "wrapper.h" 
 * Outputs      : refer "wrapper.h" 
 * Assumptions  : expects input and output to be word aligned and an exact 
 *                number of words in length 
 * ------------------------------------------------------------------------- */
void encode(pBaseDSPCodec pConfig, unsigned char *inBuffer,
        unsigned char *outBuffer)
{
    uint32_t *pDst = (uint32_t *) outBuffer;
    uint32_t *pSrc = (uint32_t *) inBuffer;
    uint32_t togo = pConfig->parent.inputBufferSize >> 2;
    pConfig->parent.outputBufferUsedSize = pConfig->parent.inputBufferSize;
    while (togo-- > 0)
        *pDst++ = *pSrc++;
}

/* ----------------------------------------------------------------------------
 * Function     : decode
 * ----------------------------------------------------------------------------
 * Description  : Implements the decode method defined in "wrapper.h" 
 *                Specifically, as this is an echo codec, input == output, 
 *                we can reuse the encode functionality for this 
 * Inputs       : refer "wrapper.h" 
 * Outputs      : refer "wrapper.h" 
 * Assumptions  : refer "wrapper.h" 
 * ------------------------------------------------------------------------- */
void decode(pBaseDSPCodec pConfig, unsigned char *inBuffer,
        unsigned char *outBuffer, bool /* unused */)
{
    encode(pConfig, inBuffer, outBuffer);
}
