/*
  ============================================================================
   File: G722.H                                  v3.0 - 10/Jan/2007
  ============================================================================

                            UGST/ITU-T G722 MODULE

                          GLOBAL FUNCTION PROTOTYPES

  History:
14.Mar.95  v1.0       Released for use ITU-T UGST software package Tool
                      based on the CNET's 07/01/90 version 2.00
01.Jul.95  v2.0       Changed function declarations to work with many compilers;
                      reformated <simao@ctd.comsat.com>
10.Jan.07  v3.0       Updated with STL2005 v2.1 basic operators
                      <{balazs.kovesi,stephane.ragot}@orange-ftgroup.com>
Porting on LPDSP32 - Prithwee Narayana Bhat  
24.Nov.15 - LPDSP32 optimizations by Ivo Coenen:
- (32-bit) int for Word16 type instead of (16-bit) short
- mult_long multiplications in qmf_tx and qmf_rx functions
- indexing instead of pointer incrementing inside loops to get dualld instructions
- using circular buffers for FIR delay lines
- static inline functions
- hardware saturation in qmf_tx and qmf_rx functions
- shifting inside limit function
- binary search in quantl function
- numerous minor optimizations
- input/output code is unsigned char type for easier handling
  ============================================================================
*/
#ifndef G722_H
#define G722_H 200

/* DEFINITION FOR SMART PROTOTYPES */
#ifndef ARGS
#if (defined(__STDC__) || defined(VMS) || defined(__DECC)  || defined(MSDOS) || defined(__MSDOS__)) || defined (__CYGWIN__) || defined (_MSC_VER)
#define ARGS(x) x
#else /* Unix: no parameters in prototype! */
#define ARGS(x) ()
#endif
#endif

/* Include function prototypes for G722 operators and type definitions */
#include "operg722.h"
#include "stl.h"

/* Define type for G.722 state structure */
typedef struct
{
  Word16          al[3];
  Word16          bl[7];
  Word16          detl;
  Word16          dlt[7]; /* dlt[0]=dlt */
  Word16          plt[3]; /* plt[0]=plt */
  Word16          rlt[3];
  Word16          nbl;
	
  Word16          ah[3];
  Word16          bh[7];
  Word16          deth;
  Word16          dh[7]; /* dh[0]=dh */
  Word16          ph[3]; /* ph[0]=ph */
  Word16          rh[3];
  Word16          nbh;
  
  Word16          sl;
  Word16          spl;
  Word16          szl;
  
  Word16          sh;
  Word16          szh;
  
  Word16          qmf_tx_delayx[24];
  Word16          *p_tx_delayx;
  Word16          qmf_rx_delayx[24];
  Word16          *p_rx_delayx;
} g722_state;

/* Include function prototypes for G722 functions */

#ifdef FLATTEN_LOOPS
#define ENCODE_LOOP_LEVELS loop_levels_1
#define DECODE_LOOP_LEVELS loop_levels_1
#else
#define ENCODE_LOOP_LEVELS loop_levels_2
#define DECODE_LOOP_LEVELS loop_levels_2
#endif

/* High-level (UGST) function prototypes for G722 functions */
void g722_reset_encoder ARGS((g722_state *encoder)) property(loop_levels_1);
Word32 g722_encode ARGS((short *incode, unsigned char *code, int nsmp, 
		       g722_state *encoder, int bps)) property(ENCODE_LOOP_LEVELS);
void g722_reset_decoder ARGS((g722_state *decoder)) property(loop_levels_1);
short g722_decode ARGS((unsigned char *code, short *outcode, int mode, 
		       int nsmp, g722_state *decoder)) property(DECODE_LOOP_LEVELS);

#endif /* G722_H */
/* ................. End of file g722.h .................................. */
